# Copyright (c) 2016, Patrick Uiterwijk <patrick@puiterwijk.org>
# All rights reserved.
#
# This file is part of Basset.
#
# Basset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Basset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Basset.  If not, see <http://www.gnu.org/licenses/>.

import logging

log = logging.getLogger(__name__)

from fedora.client.fas2 import AccountSystem
from pymongo import MongoClient
from bson.objectid import ObjectId

from basset.score.details import DetailSanityScore
from basset.score.ip import IPScore
from basset.decision.fas import FASRegisterDecision
from basset.decision.fas import FASCLADecision

PLUGINS = {
    'score': [
        DetailSanityScore,
        IPScore,
    ],
    'decision': [
        FASRegisterDecision,
        FASCLADecision,
    ]
    }

class Core(object):

    def __init__(self, config):
        log.debug('Initializing Core')

        self.config = config

        self.score_results = {
            'manual': self.config.getint('core', 'registration_check_manually'),
            'deny': self.config.getint('core', 'registration_deny')
        }

        self.plugins = {}
        self._init_plugins()
        self._init_clients()
    
    def _to_lowercase_dict(self, dct):
        if not isinstance(dct, dict):
            return dct
        else:
            return {k.lower(): self._to_lowercase_dict(dct[k])
                    for k in dct}

    def _init_clients(self):
        log.debug('Initializing clients')
        log.debug('FAS Client start')
        self.fasclient = AccountSystem(base_url=self.config.get('core', 'fasclient_url'),
                                       username=self.config.get('core', 'fasclient_username'),
                                       password=self.config.get('core', 'fasclient_password'),
                                       debug=True)
        log.debug('Testing FAS client')
        # Just a test to see if we can login
        self.fasclient.person_by_username(self.config.get('core', 'fasclient_username'))
        log.debug('FAS Client ready')
        log.debug('Mongo Client start')
        self.mongoclient = MongoClient(self.config.get('mongo', 'host'),
                                       int(self.config.get('mongo', 'port')))
        self.mongodb = self.mongoclient[self.config.get('mongo', 'dbname')]
        log.debug('Testing mongo client')
        # This will make us crash on start if we can't connect,
        # which is a better failure than on first processing.
        self.mongodb['messages'].find_one()
        log.debug('Mongo client ready')
        log.debug('Finished client initialization')

    def _init_plugins(self):
        log.debug('Initializing plugins')
        for plugin_type in PLUGINS:
            log.debug('%s plugins' % plugin_type)
            self.plugins[plugin_type] = []
            for plugin in PLUGINS[plugin_type]:
                log.debug('Loading %s' % plugin)
                self.plugins[plugin_type].append(plugin(self, self.config))
        log.debug('Finished loading plugins')

    def _get_spam_scores(self, action, time, message, quickonly=True):
        scores = {}
        for plugin in self.plugins['score']:
            # Skip non-quick modules if we were asked for quick
            if not quickonly or plugin.quick:
                scores.update(plugin.get_score(action, time, message))
        return scores

    def store_message(self, action, time, data):
        return str(self.mongodb['messages'].insert({
            'action': action,
            'time': time,
            'data': data}))

    def store_decisions(self, mid, decisions):
        inserted = []
        for decision in decisions:
            inserted.append(str(self.mongodb['decisions'].insert({
                'mid': ObjectId(mid),
                'decision': decision})))
        return inserted

    def process_fas_message(self, action, time, data):
        scores = self._get_spam_scores(action, time, data, quickonly=False)
        log.info('Scores: %s' % scores)

        score = sum(scores.values())
        log.info('Total score: %d' % score)

        if score > self.score_results['deny']:
            log.warning('Decided to deny request!')
            return {'action': action,
                    'username': data['user']['username'],
                    'decision': 'deny'}
        elif score > self.score_results['manual']:
            log.warning('Decided to move request to manual!')
            return {'action': action,
                    'username': data['user']['username'],
                    'decision': 'manual'}
        else:
            log.info('Accepted request')
            return {'action': action,
                    'username': data['user']['username'],
                    'decision': 'accept'}

    def process_message(self, action, time, data):
        log.info('Received message with action %s at %s containing %s' %
                  (action, time, data))
        data = self._to_lowercase_dict(data)
        log.debug('Lower-cased data')

        if action in ['fedora.fas.registration', 'fedora.fas.cla_sign']:
            return self.process_fas_message(action, time, data)
        else:
            log.error('Unknown action submitted')
            return []

    def process_decision(self, decision):
        log.info('Processing decision %s' % decision)

        handled = False
        for plugin in self.plugins['decision']:
            log.debug('Checking if %s handles it' % plugin)
            if plugin.handles(decision['action']):
                log.debug('Yes, requesting handling')
                return plugin.handle(decision)
        
        log.warning('Nothing handled this decision?')

    def process_decisions(self, decisions):
        if isinstance(decisions, dict):
            # If we have just one, we have just one
            decisions = [decisions]

        log.info('Decision set: %s' % decisions)

        results = []
        for decision in decisions:
            results.append(self.process_decision(decision))

        log.info('Processing results: %s' % results)
        if False in results:
            return False
        else:
            return True
