# Copyright (c) 2016, Patrick Uiterwijk <patrick@puiterwijk.org>
# All rights reserved.
#
# This file is part of Basset.
#
# Basset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Basset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Basset.  If not, see <http://www.gnu.org/licenses/>.

import logging

from basset.decision.base import DecisionPlugin


class FASRegisterDecision(DecisionPlugin):
    name = 'fasregistration'
    handled_decisions = ['fedora.fas.registration']

    def _update_account_status(self, username, new_status):
        self.log.info('Updating status for %s to %s' % (username, new_status))
        res = self.core.fasclient.send_request('user/updatestatus/%s' % username,
                                               auth=True,
                                               req_params={'status': new_status})
        msg = res.get('tg_flash', None)
        if msg is None:
            msg = ''
        if not "You're not allowed to update accounts!" in msg and \
                not "Account status could not be changed:" in msg:
            return True
        else:
            self.log.error('Error saving status: %s' % res.get('tg_flash', ''))
            return False

    def _accept_account(self, username):
        self.log.info("Accepting new user: %s" % username)
        res = self.core.fasclient.send_request('user/acceptuser/%s' % username,
                                               auth=True,
                                               req_params={'status': ''})
        if not res.get('result', ''):
            self.log.error('Unable to accept: %s' % res)
            return False
        else:
            return True

    def handle(self, decision):
        if decision['decision'] == 'deny':
            return self._update_account_status(decision['username'],
                                               'spamcheck_denied')
        elif decision['decision'] == 'manual':
            return self._update_account_status(decision['username'],
                                               'spamcheck_manual')
        elif decision['decision'] == 'accept':
            return self._accept_account(decision['username'])
        else:
            raise ValueError('Invalid decision value: "%s"' % decision['decision'])

class FASCLADecision(DecisionPlugin):
    name = 'fascla'
    handled_decisions = ['fedora.fas.cla_sign']

    def _remove_from_group(self, group, username):
        self.log.info('Removing %s from %s' % (username, group))
        res = self.core.fasclient.send_request('group/remove/%s/%s' % (group, username),
                                               auth=True)
        res = res.get('tg_flash', '')
        if not 'has been removed from' in res:
            self.log.error('Error removing user: %s' % res)
            return False
        else:
            return True

    def _sponsor_into_group(self, group, username):
        self.log.info('Sponsoring %s into %s' % (username, group))
        res = self.core.fasclient.send_request('group/sponsor/%s/%s' % (group, username),
                                               auth=True)
        res = res.get('tg_flash', '')
        if not 'has been sponsored' in res:
            self.log.error('Error removing user: %s' % res)
            return False
        else:
            return True

    def handle(self, decision):
        cla_group = 'cla_fpca'
        if decision['decision'] == 'deny':
            return self._remove_from_group(cla_group, decision['username'])
        elif decision['decision'] == 'accept':
            # TODO: Send email to legal
            return self._sponsor_into_group(cla_group, decision['username'])
        elif decision['decision'] == 'manual':
            # TODO: Send email to admins
            self.log.error('Need to handle manually')
            return False
        else:
            raise ValueError('Invalid decision value: "%s"' % decision['decision'])
