# Copyright (c) 2016, Patrick Uiterwijk <patrick@puiterwijk.org>
# All rights reserved.
#
# This file is part of Basset.
#
# Basset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Basset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Basset.  If not, see <http://www.gnu.org/licenses/>.

import Levenshtein

from basset.score.base import ScorePlugin


class DetailSanityScore(ScorePlugin):
    name = 'details'
    quick = True

    def get_score(self, action, time, data):
        username = data['user']['username']
        email = data['user']['email']
        human_name = data['user']['human_name']
        country = data['user']['country_code']
        phone = data['user']['telephone']

        scores = {}
        scores['gpg_ssh_irc'] = self.get_gpg_ssh_irc_score(data['user'])
        scores['username_numbers'] = self.get_username_numbers_score(username)
        scores['email_numbers'] = self.get_email_numbers_score(username, email)
        scores['fullname_username'] = self.get_fullname_username_score(username, human_name)
        scores['email_domain'] = self.get_email_domain_scores(email)
        scores['email'] = self.get_email_scores(username, email, human_name)
        scores['country'] = self.get_country_scores(country)
        #scores['phone'] = self.get_phone_scores(country, phone)

        return scores

    def get_gpg_ssh_irc_score(self, user):
        minscore = 0
        for data in ['gpg_keyid', 'ssh_key', 'ircnick']:
            if data in user:
                minscore -= int(self.config['gpg_ssh_irc_minscore'])
        return minscore

    def get_username_numbers_score(self, username):
        score = sum(c.isdigit() for c in username)

        return int(int(score) * float(self.config['username_number_multiplier']))

    def get_email_numbers_score(self, username, email):
        nr_name = sum(c.isdigit() for c in username)
        nr_email = sum(c.isdigit() for c in email)
        # Don't count numbers that were in the username twice
        return int(max(0, int((nr_email - nr_name))) * float(self.config['email_number_multiplier']))

    def get_fullname_username_score(self, username, human_name):
        dif = Levenshtein.ratio(username, human_name)
        if dif >= 0.9:
            return self.config['fullname_username_alike_very']
        elif dif >= 0.8:
            return self.config['fullname_username_alike_alot']
        else:
            return 0

    def get_email_domain_scores(self, email):
        domain = email[email.find('@')+1:]
        mapping = {}
        for option in self.config:
            if option.startswith('email_domain_'):
                mapping[option[len('email_domain_'):]] = self.config[option]
        if domain in mapping:
            return int(mapping[domain])
        else:
            return 0

    def get_email_scores(self, username, email, human_name):
        email = email[:email.find('@')]
        dif = Levenshtein.ratio(human_name, email)
        if dif >= 0.9:
            return int(self.config['fullname_email_alike_very'])
        elif dif >= 0.8:
            return int(self.config['fullname_email_alike_alot'])
        else:
            return 0

    def get_country_scores(self, country):
        mapping = {}
        for option in self.config:
            if option.startswith('country_'):
                mapping[option[len('country_'):]] = self.config[option]
        if country in mapping:
            return int(mapping[country])
        else:
            return 0
